/**
* Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */
#pragma once

namespace smithy {
extern SMITHY_API const char *SIGNER_REGION_PROPERTY;
extern SMITHY_API const char *SIGNER_SERVICE_NAME;
}
